from typing import Tuple, Optional

from pygame.rect import Rect
from pygame.surface import Surface

from state import World
from state.constants import LAYER_GROUND_EARTH, LAYER_GROUND_SEA
from ui.MouseButtons import MouseButtons
from ui.Theme import Theme
from ui.mode.GameMode import GameMode


class EditGameMode(GameMode):

    def __init__(self, theme: Theme, world: World):
        super().__init__(theme)
        self.__world = world
        self.__mouseButtonDown = False

    # Game loop

    def processInput(self):
        pass

    def update(self):
        pass

    def render(self, surface: Surface):
        theme = self.theme
        tileWidth = theme.tileWidth
        tileHeight = theme.tileHeight
        tiles = theme.tiles
        tileset = theme.tileset
        for y in range(self.__world.height):
            for x in range(self.__world.width):
                tile = tiles[self.__world.getValue(x, y)]
                tileRect = Rect(
                    tile[0] * tileWidth, tile[1] * tileHeight,
                    tileWidth, tileHeight
                )
                tileCoords = (
                    x * tileWidth,
                    y * tileHeight
                )
                surface.blit(tileset, tileCoords, tileRect)

    # Mouse handling

    def __computeCellCoordintates(self, mouseX: int, mouseY: int) -> Optional[Tuple[int, int]]:
        cellX = mouseX // self.theme.tileWidth
        cellY = mouseY // self.theme.tileHeight
        if not (0 <= cellX < self.__world.width) \
        or not (0 <= cellY < self.__world.height):
            return None
        return cellX, cellY

    def __updateCell(self, cellX: int, cellY: int, buttons: MouseButtons):
        if buttons.button1:
            self.__world.setValue(cellX, cellY, LAYER_GROUND_EARTH)
        elif buttons.button3:
            self.__world.setValue(cellX, cellY, LAYER_GROUND_SEA)

    def mouseButtonDown(self, mouseX: int, mouseY: int, buttons: MouseButtons):
        coords = self.__computeCellCoordintates(mouseX, mouseY)
        if coords is None:
            return
        cellX, cellY = coords
        self.__mouseButtonDown = True
        self.__updateCell(cellX, cellY, buttons)

    def mouseMove(self, mouseX: int, mouseY: int, buttons: MouseButtons):
        if not self.__mouseButtonDown:
            return
        coords = self.__computeCellCoordintates(mouseX, mouseY)
        if coords is None:
            return
        cellX, cellY = coords
        self.__updateCell(cellX, cellY, buttons)

    def mouseButtonUp(self, mouseX: int, mouseY: int, buttons: MouseButtons):
        self.__mouseButtonDown = False

    def mouseEnter(self, mouseX: int, mouseY: int, buttons: MouseButtons):
        self.__mouseButtonDown = False

    def mouseLeave(self):
        self.__mouseButtonDown = False
