from abc import ABC, abstractmethod

from pygame.surface import Surface

from ui.MouseButtons import MouseButtons
from ui.MouseWheel import MouseWheel
from ui.Theme import Theme


class GameMode(ABC):

    def __init__(self, theme: Theme):
        self.__theme = theme

    @property
    def theme(self):
        return self.__theme

    @abstractmethod
    def processInput(self):
        raise NotImplementedError()

    @abstractmethod
    def update(self):
        raise NotImplementedError()

    @abstractmethod
    def render(self, surface: Surface):
        raise NotImplementedError()

    # Mouse handling

    def mouseEnter(self, mouseX: int, mouseY: int, buttons: MouseButtons):
        return False

    def mouseLeave(self):
        return False

    def mouseButtonDown(self, mouseX: int, mouseY: int, buttons: MouseButtons):
        return False

    def mouseButtonUp(self, mouseX: int, mouseY: int, buttons: MouseButtons):
        return False

    def mouseWheel(self, mouseX: int, mouseY: int, buttons: MouseButtons, wheel: MouseWheel):
        return False

    def mouseMove(self, mouseX: int, mouseY: int, buttons: MouseButtons):
        return False